import { isMatchCondition } from './match-condition';
export const deleteValue = Symbol('deleteValue');
export function deepAssign(currentObj, newObj) {
    let _newObj = { ...newObj };
    if (!newObj)
        return currentObj;
    for (const [key, value] of Object.entries(_newObj)) {
        if (!currentObj[key]) {
            continue;
        }
        if (typeof value == 'object' && value && value.constructor.name === 'Object') {
            _newObj[key] = deepAssign(currentObj[key], _newObj[key]);
        }
    }
    return { ...currentObj, ..._newObj };
}
export function applyMutation(conditions, obj, option = {}) {
    const defaultOption = { keepMutation: false, top: true };
    Object.assign(defaultOption, option);
    let result = { ...obj };
    let lastObjResult = result;
    if (result.$mutate) {
        for (let i = 0; i < result.$mutate.length; i++) {
            const mutateObj = result.$mutate[i];
            const { cond } = mutateObj;
            if (isMatchCondition(conditions.map((c) => (typeof c == 'object' ? c.condition : c)), cond)) {
                const mutation = lastObjResult.$mutate[i]['mutation'];
                if (mutation === deleteValue) {
                    result = deleteValue;
                    break;
                }
                else if (typeof mutation == 'function') {
                    result = mutation.bind(this)(result, { conditions });
                }
                else {
                    result = deepAssign(lastObjResult, mutation);
                }
                if (result && typeof result == 'object')
                    lastObjResult = result;
            }
        }
    }
    for (const [key, value] of Object.entries(result)) {
        if (key == '$mutate')
            continue;
        if (value && typeof value == 'object' && value.constructor.name === 'Object') {
            result[key] = applyMutation.bind(this)(conditions, value, { ...option, top: false });
        }
        if (result[key] == deleteValue)
            delete result[key];
    }
    if (!option.keepMutation) {
        if (typeof result == 'object' && '$mutate' in result)
            delete result['$mutate'];
    }
    if (option.top && result == deleteValue)
        result = undefined;
    return result;
}
